@@ -28,6 +28,14 @@ class SimpleSAML_Memcache
     private static $serverGroups = null;
 
 
+  /**
+   * The flavor of memcache PHP extension we are using.
+   *
+   * @var string
+   */
+    private static $extension = '';
+
+
     /**
      * Find data stored with a given key.
      *
@@ -37,7 +45,7 @@ class SimpleSAML_Memcache
      */
     public static function get($key)
     {
-        SimpleSAML_Logger::debug("loading key $key from memcache");
+        SimpleSAML\Logger::debug("loading key $key from memcache");
 
         $latestInfo = null;
         $latestTime = 0.0;
@@ -68,19 +76,19 @@ class SimpleSAML_Memcache
              * - 'data': The data.
              */
             if (!is_array($info)) {
-                SimpleSAML_Logger::warning(
+                SimpleSAML\Logger::warning(
                     'Retrieved invalid data from a memcache server. Data was not an array.'
                 );
                 continue;
             }
             if (!array_key_exists('timestamp', $info)) {
-                SimpleSAML_Logger::warning(
+                SimpleSAML\Logger::warning(
                     'Retrieved invalid data from a memcache server. Missing timestamp.'
                 );
                 continue;
             }
             if (!array_key_exists('data', $info)) {
-                SimpleSAML_Logger::warning(
+                SimpleSAML\Logger::warning(
                     'Retrieved invalid data from a memcache server. Missing data.'
                 );
                 continue;
@@ -117,13 +125,13 @@ class SimpleSAML_Memcache
                 throw new SimpleSAML_Error_Exception('All memcache servers are down', 503, $e);
             }
             // we didn't find any data matching the key
-            SimpleSAML_Logger::debug("key $key not found in memcache");
+            SimpleSAML\Logger::debug("key $key not found in memcache");
             return null;
         }
 
         if ($mustUpdate) {
             // we found data matching the key, but some of the servers need updating
-            SimpleSAML_Logger::debug("Memcache servers out of sync for $key, forcing sync");
+            SimpleSAML\Logger::debug("Memcache servers out of sync for $key, forcing sync");
             self::set($key, $latestData);
         }
 
@@ -140,7 +148,7 @@ class SimpleSAML_Memcache
      */
     public static function set($key, $value, $expire = null)
     {
-        SimpleSAML_Logger::debug("saving key $key to memcache");
+        SimpleSAML\Logger::debug("saving key $key to memcache");
         $savedInfo = array(
             'timestamp' => microtime(true),
             'data'      => $value
@@ -154,7 +162,12 @@ class SimpleSAML_Memcache
 
         // store this object to all groups of memcache servers
         foreach (self::getMemcacheServers() as $server) {
-            $server->set($key, $savedInfoSerialized, 0, $expire);
+            if (self::$extension === 'memcached') {
+                $server->set($key, $savedInfoSerialized, $expire);
+            }
+            else {
+                $server->set($key, $savedInfoSerialized, 0, $expire);
+            }
         }
     }
 
@@ -167,7 +180,7 @@ class SimpleSAML_Memcache
     public static function delete($key)
     {
         assert('is_string($key)');
-        SimpleSAML_Logger::debug("deleting key $key from memcache");
+        SimpleSAML\Logger::debug("deleting key $key from memcache");
 
         // store this object to all groups of memcache servers
         foreach (self::getMemcacheServers() as $server) {
@@ -277,7 +290,11 @@ class SimpleSAML_Memcache
         }
 
         // add this server to the Memcache object
-        $memcache->addServer($hostname, $port, true, $weight, $timeout, $timeout, true);
+        if (self::$extension === 'memcached') {
+            $memcache->addServer($hostname, $port);
+        } else {
+            $memcache->addServer($hostname, $port, true, $weight, $timeout, $timeout, true);
+        }
     }
 
 
@@ -293,12 +310,14 @@ class SimpleSAML_Memcache
      */
     private static function loadMemcacheServerGroup(array $group)
     {
-        if (!class_exists('Memcache')) {
-            throw new Exception('Missing Memcache class. Is the memcache extension installed?');
+        $class = class_exists('Memcache') ? 'Memcache' : (class_exists('Memcached') ? 'Memcached' : FALSE);
+        if (!$class) {
+            throw new Exception('Missing Memcached implementation. You must install either the Memcache or Memcached extension.');
         }
+        self::$extension = strtolower($class);
 
         // create the Memcache object
-        $memcache = new Memcache();
+        $memcache = new $class();
 
         // iterate over all the servers in the group and add them to the Memcache object
         foreach ($group as $index => $server) {
