@@ -1,6 +1,8 @@
 <?php
 namespace SimpleSAML\Utils;
 
+use SimpleSAML\Module;
+use SimpleSAML\Logger;
 
 /**
  * HTTP-related utility methods.
@@ -16,6 +18,7 @@ class HTTP
      * @param string $destination The destination URL.
      * @param array  $data An associative array containing the data to be posted to $destination.
      *
+     * @throws \SimpleSAML_Error_Exception If the current session is transient.
      * @return string  A URL which allows to securely post a form to $destination.
      *
      * @author Jaime Perez, UNINETT AS <jaime.perez@uninett.no>
@@ -25,10 +28,17 @@ class HTTP
         $session = \SimpleSAML_Session::getSessionFromRequest();
         $id = self::savePOSTData($session, $destination, $data);
 
-        // encrypt the session ID and the random ID
-        $info = base64_encode(Crypto::aesEncrypt($session->getSessionId().':'.$id));
+        // get the session ID
+        $session_id = $session->getSessionId();
+        if (is_null($session_id)) {
+            // this is a transient session, it is pointless to continue
+            throw new \SimpleSAML_Error_Exception('Cannot save POST data to a transient session.');
+        }
 
-        $url = \SimpleSAML_Module::getModuleURL('core/postredirect.php', array('RedirInfo' => $info));
+        // encrypt the session ID and the random ID
+        $info = base64_encode(Crypto::aesEncrypt($session_id.':'.$id));
+
+        $url = Module::getModuleURL('core/postredirect.php', array('RedirInfo' => $info));
         return preg_replace('#^https:#', 'http:', $url);
     }
 
@@ -71,7 +81,7 @@ class HTTP
      *
      * @author Olav Morken, UNINETT AS <olav.morken@uninett.no>
      */
-    private static function getServerHTTPS()
+    public static function getServerHTTPS()
     {
         if (!array_key_exists('HTTPS', $_SERVER)) {
             // not an https-request
@@ -83,8 +93,8 @@ class HTTP
             return false;
         }
 
-        // otherwise, HTTPS will be a non-empty string
-        return $_SERVER['HTTPS'] !== '';
+        // otherwise, HTTPS will be non-empty
+        return !empty($_SERVER['HTTPS']);
     }
 
 
@@ -96,7 +106,7 @@ class HTTP
      *
      * @author Olav Morken, UNINETT AS <olav.morken@uninett.no>
      */
-    private static function getServerPort()
+    public static function getServerPort()
     {
         $port = (isset($_SERVER['SERVER_PORT'])) ? $_SERVER['SERVER_PORT'] : '80';
         if (self::getServerHTTPS()) {
@@ -157,15 +167,17 @@ class HTTP
         }
 
         if (strlen($url) > 2048) {
-            \SimpleSAML_Logger::warning('Redirecting to a URL longer than 2048 bytes.');
+            Logger::warning('Redirecting to a URL longer than 2048 bytes.');
         }
 
-        // set the location header
-        header('Location: '.$url, true, $code);
+        if (!headers_sent()) {
+            // set the location header
+            header('Location: '.$url, true, $code);
 
-        // disable caching of this response
-        header('Pragma: no-cache');
-        header('Cache-Control: no-cache, must-revalidate');
+            // disable caching of this response
+            header('Pragma: no-cache');
+            header('Cache-Control: no-cache, must-revalidate');
+        }
 
         // show a minimal web page with a clickable link to the URL
         echo '<?xml version="1.0" encoding="UTF-8"?>'."\n";
@@ -174,6 +186,7 @@ class HTTP
         echo '<html xmlns="http://www.w3.org/1999/xhtml">'."\n";
         echo "  <head>\n";
         echo '    <meta http-equiv="content-type" content="text/html; charset=utf-8">'."\n";
+        echo '    <meta http-equiv="refresh" content="0;URL=\''.htmlspecialchars($url).'\'">'."\n";
         echo "    <title>Redirect</title>\n";
         echo "  </head>\n";
         echo "  <body>\n";
@@ -242,6 +255,7 @@ class HTTP
             $oldQuery = array();
             $url .= '?';
         } else {
+            /** @var string|false $oldQuery */
             $oldQuery = substr($url, $queryStart + 1);
             if ($oldQuery === false) {
                 $oldQuery = array();
@@ -251,6 +265,7 @@ class HTTP
             $url = substr($url, 0, $queryStart + 1);
         }
 
+        /** @var array $oldQuery */
         $query = array_merge($oldQuery, $parameters);
         $url .= http_build_query($query, '', '&');
 
@@ -261,7 +276,7 @@ class HTTP
     /**
      * Check for session cookie, and show missing-cookie page if it is missing.
      *
-     * @param string|NULL $retryURL The URL the user should access to retry the operation. Defaults to null.
+     * @param string|null $retryURL The URL the user should access to retry the operation. Defaults to null.
      *
      * @return void If there is a session cookie, nothing will be returned. Otherwise, the user will be redirected to a
      *     page telling about the missing cookie.
@@ -271,7 +286,7 @@ class HTTP
      */
     public static function checkSessionCookie($retryURL = null)
     {
-        if (!is_string($retryURL) && !is_null($retryURL)) {
+        if (!is_null($retryURL) && !is_string($retryURL)) {
             throw new \InvalidArgumentException('Invalid input parameters.');
         }
 
@@ -282,7 +297,7 @@ class HTTP
 
         // we didn't have a session cookie. Redirect to the no-cookie page
 
-        $url = \SimpleSAML_Module::getModuleURL('core/no_cookie.php');
+        $url = Module::getModuleURL('core/no_cookie.php');
         if ($retryURL !== null) {
             $url = self::addURLParameters($url, array('retryURL' => $retryURL));
         }
@@ -311,6 +326,10 @@ class HTTP
         }
         $url = self::normalizeURL($url);
 
+        if (filter_var($url, FILTER_VALIDATE_URL) === false) {
+            throw new \SimpleSAML_Error_Exception('Invalid URL: '.$url);
+        }
+
         // get the white list of domains
         if ($trustedSites === null) {
             $trustedSites = \SimpleSAML_Configuration::getInstance()->getValue('trusted.url.domains', array());
@@ -329,12 +348,30 @@ class HTTP
                 $hostname = $hostname.$matches[3];
             }
 
-            // add self host to the white list
-            $self_host = self::getSelfHost();
-            $trustedSites[] = $self_host;
+            $self_host = self::getSelfHostWithNonStandardPort();
+
+            $trustedRegex = \SimpleSAML_Configuration::getInstance()->getValue('trusted.url.regex', false);
+
+            $trusted = false;
+            if ($trustedRegex) {
+                // add self host to the white list
+                $trustedSites[] = preg_quote($self_host);
+                foreach ($trustedSites as $regex) {
+                    // Add start and end delimiters.
+                    $regex = "@^{$regex}$@";
+                    if (preg_match($regex, $hostname)) {
+                        $trusted = true;
+                        break;
+                    }
+                }
+            } else {
+                // add self host to the white list
+                $trustedSites[] = $self_host;
+                $trusted = in_array($hostname, $trustedSites, true);
+            }
 
             // throw exception due to redirection to untrusted site
-            if (!in_array($hostname, $trustedSites)) {
+            if (!$trusted) {
                 throw new \SimpleSAML_Error_Exception('URL not allowed: '.$url);
             }
         }
@@ -343,7 +380,8 @@ class HTTP
 
 
     /**
-     * Helper function to retrieve a file or URL with proxy support.
+     * Helper function to retrieve a file or URL with proxy support, also
+     * supporting proxy basic authorization..
      *
      * An exception will be thrown if we are unable to retrieve the data.
      *
@@ -351,7 +389,8 @@ class HTTP
      * @param array   $context Extra context options. This parameter is optional.
      * @param boolean $getHeaders Whether to also return response headers. Optional.
      *
-     * @return mixed array if $getHeaders is set, string otherwise
+     * @return string|array An array if $getHeaders is set, containing the data and the headers respectively; string
+     *  otherwise.
      * @throws \InvalidArgumentException If the input parameters are invalid.
      * @throws \SimpleSAML_Error_Exception If the file or URL cannot be retrieved.
      *
@@ -372,6 +411,10 @@ class HTTP
             if (!isset($context['http']['proxy'])) {
                 $context['http']['proxy'] = $proxy;
             }
+            $proxy_auth = $config->getString('proxy.auth', false);
+            if ($proxy_auth !== false) {
+                $context['http']['header'] = "Proxy-Authorization: Basic".base64_encode($proxy_auth);
+            }
             if (!isset($context['http']['request_fulluri'])) {
                 $context['http']['request_fulluri'] = true;
             }
@@ -395,7 +438,7 @@ class HTTP
                         'SNI_enabled'     => true,
                     );
                 } else {
-                    \SimpleSAML_Logger::warning('Invalid URL format or local URL used through a proxy');
+                    Logger::warning('Invalid URL format or local URL used through a proxy');
                 }
             }
         }
@@ -404,7 +447,8 @@ class HTTP
         $data = file_get_contents($url, false, $context);
         if ($data === false) {
             $error = error_get_last();
-            throw new \SimpleSAML_Error_Exception('Error fetching '.var_export($url, true).':'.$error['message']);
+            throw new \SimpleSAML_Error_Exception('Error fetching '.var_export($url, true).':'.
+                (is_array($error) ? $error['message'] : 'no error available'));
         }
 
         // data and headers
@@ -501,12 +545,45 @@ class HTTP
     }
 
 
+    /**
+     * Try to guess the base SimpleSAMLphp path from the current request.
+     *
+     * This method offers just a guess, so don't rely on it.
+     *
+     * @return string The guessed base path that should correspond to the root installation of SimpleSAMLphp.
+     */
+    public static function guessBasePath()
+    {
+        if (!array_key_exists('REQUEST_URI', $_SERVER) || !array_key_exists('SCRIPT_FILENAME', $_SERVER)) {
+            return '/';
+        }
+        // get the name of the current script
+        $path = explode('/', $_SERVER['SCRIPT_FILENAME']);
+        $script = array_pop($path);
+
+        // get the portion of the URI up to the script, i.e.: /simplesaml/some/directory/script.php
+        if (!preg_match('#^/(?:[^/]+/)*'.$script.'#', $_SERVER['REQUEST_URI'], $matches)) {
+            return '/';
+        }
+        $uri_s = explode('/', $matches[0]);
+        $file_s = explode('/', $_SERVER['SCRIPT_FILENAME']);
+
+        // compare both arrays from the end, popping elements matching out of them
+        while ($uri_s[count($uri_s) - 1] === $file_s[count($file_s) - 1]) {
+            array_pop($uri_s);
+            array_pop($file_s);
+        }
+        // we are now left with the minimum part of the URI that does not match anything in the file system, use it
+        return join('/', $uri_s).'/';
+    }
+
+
     /**
      * Retrieve the base URL of the SimpleSAMLphp installation. The URL will always end with a '/'. For example:
      *      https://idp.example.org/simplesaml/
      *
      * @return string The absolute base URL for the SimpleSAMLphp installation.
-     * @throws \SimpleSAML_Error_Exception If 'baseurlpath' has an invalid format.
+     * @throws \SimpleSAML\Error\CriticalConfigurationError If 'baseurlpath' has an invalid format.
      *
      * @author Olav Morken, UNINETT AS <olav.morken@uninett.no>
      */
@@ -515,9 +592,9 @@ class HTTP
         $globalConfig = \SimpleSAML_Configuration::getInstance();
         $baseURL = $globalConfig->getString('baseurlpath', 'simplesaml/');
 
-        if (preg_match('#^https?://.*/$#D', $baseURL, $matches)) {
+        if (preg_match('#^https?://.*/?$#D', $baseURL, $matches)) {
             // full URL in baseurlpath, override local server values
-            return $baseURL;
+            return rtrim($baseURL, '/').'/';
         } elseif (
             (preg_match('#^/?([^/]?.*/)$#D', $baseURL, $matches)) ||
             (preg_match('#^\*(.*)/$#D', $baseURL, $matches)) ||
@@ -530,14 +607,22 @@ class HTTP
 
             $hostname = self::getServerHost();
             $port = self::getServerPort();
-            $path = '/'.$globalConfig->getBaseURL();
+            $path = $globalConfig->getBasePath();
 
             return $protocol.$hostname.$port.$path;
         } else {
-            throw new \SimpleSAML_Error_Exception('Invalid value for \'baseurlpath\' in '.
-                'config.php. Valid format is in the form: '.
-                '[(http|https)://(hostname|fqdn)[:port]]/[path/to/simplesaml/]. '.
-                'It must end with a \'/\'.');
+            /*
+             * Invalid 'baseurlpath'. We cannot recover from this, so throw a critical exception and try to be graceful
+             * with the configuration. Use a guessed base path instead of the one provided.
+             */
+            $c = $globalConfig->toArray();
+            $c['baseurlpath'] = self::guessBasePath();
+            throw new \SimpleSAML\Error\CriticalConfigurationError(
+                'Invalid value for \'baseurlpath\' in config.php. Valid format is in the form: '.
+                '[(http|https)://(hostname|fqdn)[:port]]/[path/to/simplesaml/]. It must end with a \'/\'.',
+                null,
+                $c
+            );
         }
     }
 
@@ -587,7 +672,7 @@ class HTTP
         } else { // post the data directly
             $session = \SimpleSAML_Session::getSessionFromRequest();
             $id = self::savePOSTData($session, $destination, $data);
-            $url = \SimpleSAML_Module::getModuleURL('core/postredirect.php', array('RedirId' => $id));
+            $url = Module::getModuleURL('core/postredirect.php', array('RedirId' => $id));
         }
 
         return $url;
@@ -597,22 +682,42 @@ class HTTP
     /**
      * Retrieve our own host.
      *
-     * @return string The current host (with non-default ports included).
+     * E.g. www.example.com
+     *
+     * @return string The current host.
+     *
+     * @author Jaime Perez, UNINETT AS <jaime.perez@uninett.no>
+     */
+    public static function getSelfHost()
+    {
+        $decomposed = explode(':', self::getSelfHostWithNonStandardPort());
+        return array_shift($decomposed);
+    }
+
+    /**
+     * Retrieve our own host, including the port in case the it is not standard for the protocol in use. That is port
+     * 80 for HTTP and port 443 for HTTPS.
+     *
+     * E.g. www.example.com:8080
+     *
+     * @return string The current host, followed by a colon and the port number, in case the port is not standard for
+     * the protocol.
      *
      * @author Andreas Solberg, UNINETT AS <andreas.solberg@uninett.no>
      * @author Olav Morken, UNINETT AS <olav.morken@uninett.no>
      */
-    public static function getSelfHost()
+    public static function getSelfHostWithNonStandardPort()
     {
         $url = self::getBaseURL();
 
-        $start = strpos($url, '://') + 3;
-        $length = strcspn($url, '/:', $start);
+        /** @var int $colon getBaseURL() will allways return a valid URL */
+        $colon = strpos($url, '://');
+        $start = $colon + 3;
+        $length = strcspn($url, '/', $start);
 
         return substr($url, $start, $length);
     }
 
-
     /**
      * Retrieve our own host together with the URL path. Please note this function will return the base URL for the
      * current SP, as defined in the global configuration.
@@ -627,7 +732,7 @@ class HTTP
         $baseurl = explode("/", self::getBaseURL());
         $elements = array_slice($baseurl, 3 - count($baseurl), count($baseurl) - 4);
         $path = implode("/", $elements);
-        return self::getSelfHost()."/".$path;
+        return self::getSelfHostWithNonStandardPort()."/".$path;
     }
 
 
@@ -673,15 +778,26 @@ class HTTP
              *   directory of SimpleSAMLphp, the URI does not contain its relative path, and $uri_pos is false.
              *
              * It doesn't matter which one of those cases we have. We just know we can't apply our base URL to the
-             * current URI, so we need to build it back from the PHP environment.
+             * current URI, so we need to build it back from the PHP environment, unless we have a base URL specified
+             * for this case in the configuration. First, check if that's the case.
              */
-            $protocol = 'http';
-            $protocol .= (self::getServerHTTPS()) ? 's' : '';
-            $protocol .= '://';
 
-            $hostname = self::getServerHost();
-            $port = self::getServerPort();
-            return $protocol.$hostname.$port.$_SERVER['REQUEST_URI'];
+            /** @var \SimpleSAML_Configuration $appcfg */
+            $appcfg = $cfg->getConfigItem('application', null);
+            $appurl = ($appcfg instanceof \SimpleSAML_Configuration) ? $appcfg->getString('baseURL', '') : '';
+            if (!empty($appurl)) {
+                $protocol = parse_url($appurl, PHP_URL_SCHEME);
+                $hostname = parse_url($appurl, PHP_URL_HOST);
+                $port = parse_url($appurl, PHP_URL_PORT);
+                $port = !empty($port) ? ':'.$port : '';
+
+            } else { // no base URL specified for app, just use the current URL
+                $protocol = 'http';
+                $protocol .= (self::getServerHTTPS()) ? 's' : '';
+                $hostname = self::getServerHost();
+                $port = self::getServerPort();
+            }
+            return $protocol.'://'.$hostname.$port.$_SERVER['REQUEST_URI'];
         }
 
         return self::getBaseURL().$rel_path.substr($_SERVER['REQUEST_URI'], $uri_pos + strlen($url_path));
@@ -700,7 +816,10 @@ class HTTP
     public static function getSelfURLHost()
     {
         $url = self::getSelfURL();
-        $start = strpos($url, '://') + 3;
+
+        /** @var int $colon getBaseURL() will allways return a valid URL */
+        $colon = strpos($url, '://');
+        $start = $colon + 3;
         $length = strcspn($url, '/', $start) + $start;
         return substr($url, 0, $length);
     }
@@ -922,14 +1041,13 @@ class HTTP
             return $baseScheme.$url;
         }
 
-        $firstChar = substr($url, 0, 1);
-        if ($firstChar === '/') {
+        if ($url[0] === '/') {
             return $baseHost.$url;
         }
-        if ($firstChar === '?') {
+        if ($url[0] === '?') {
             return $basePath.$url;
         }
-        if ($firstChar === '#') {
+        if ($url[0] === '#') {
             return $baseQuery.$url;
         }
 
@@ -971,6 +1089,8 @@ class HTTP
      * @throws \InvalidArgumentException If any parameter has an incorrect type.
      * @throws \SimpleSAML\Error\CannotSetCookie If the headers were already sent and the cookie cannot be set.
      *
+     * @return void
+     *
      * @author Andjelko Horvat
      * @author Jaime Perez, UNINETT AS <jaime.perez@uninett.no>
      */
@@ -1066,6 +1186,8 @@ class HTTP
      *
      * @throws \InvalidArgumentException If $destination is not a string or $data is not an array.
      *
+     * @return void
+     *
      * @author Olav Morken, UNINETT AS <olav.morken@uninett.no>
      * @author Andjelko Horvat
      * @author Jaime Perez, UNINETT AS <jaime.perez@uninett.no>
