@@ -51,11 +51,11 @@ $allLinks = array(
 	'auth'       => &$links_auth,
 	'federation' => &$links_federation,
 );
-SimpleSAML_Module::callHooks('frontpage', $allLinks);
+SimpleSAML\Module::callHooks('frontpage', $allLinks);
 
 
 $metadataHosted = array();
-SimpleSAML_Module::callHooks('metadata_hosted', $metadataHosted);
+SimpleSAML\Module::callHooks('metadata_hosted', $metadataHosted);
 
 
 
@@ -78,29 +78,35 @@ if ($isadmin) {
 if ($config->getBoolean('enable.saml20-idp', FALSE) === true) {
 	try {
 		$metaentries['hosted']['saml20-idp'] = $metadata->getMetaDataCurrent('saml20-idp-hosted');
-		$metaentries['hosted']['saml20-idp']['metadata-url'] = '/' . $config->getBaseURL() .
+		$metaentries['hosted']['saml20-idp']['metadata-url'] = $config->getBasePath() .
                                                                'saml2/idp/metadata.php?output=xhtml';
 		if ($isadmin)
 			$metaentries['remote']['saml20-sp-remote'] = $metadata->getList('saml20-sp-remote');
-	} catch(Exception $e) {}
+	} catch(Exception $e) {
+        SimpleSAML_Logger::error('Federation: Error loading saml20-idp: ' . $e->getMessage());
+    }
 }
 if ($config->getBoolean('enable.shib13-idp', FALSE) === true) {
 	try {
 		$metaentries['hosted']['shib13-idp'] = $metadata->getMetaDataCurrent('shib13-idp-hosted');
-		$metaentries['hosted']['shib13-idp']['metadata-url'] = '/' . $config->getBaseURL() .
+		$metaentries['hosted']['shib13-idp']['metadata-url'] = $config->getBasePath() .
                                                                'shib13/idp/metadata.php?output=xhtml';
 		if ($isadmin)
 			$metaentries['remote']['shib13-sp-remote'] = $metadata->getList('shib13-sp-remote');
-	} catch(Exception $e) {}
+	} catch(Exception $e) {
+        SimpleSAML_Logger::error('Federation: Error loading shib13-idp: ' . $e->getMessage());
+    }
 }
 if ($config->getBoolean('enable.adfs-idp', FALSE) === true) {
     try {
         $metaentries['hosted']['adfs-idp'] = $metadata->getMetaDataCurrent('adfs-idp-hosted');
-        $metaentries['hosted']['adfs-idp']['metadata-url'] = SimpleSAML_Module::getModuleURL('adfs/idp/metadata.php',
+        $metaentries['hosted']['adfs-idp']['metadata-url'] = SimpleSAML\Module::getModuleURL('adfs/idp/metadata.php',
                                                                                              array('output' => 'xhtml'));
         if ($isadmin)
             $metaentries['remote']['adfs-sp-remote'] = $metadata->getList('adfs-sp-remote');
-    } catch(Exception $e) {}
+    } catch(Exception $e) {
+        SimpleSAML_Logger::error('Federation: Error loading adfs-idp: ' . $e->getMessage());
+    }
 }
 
 foreach ($metaentries['remote'] as $key => $value) {
@@ -109,10 +115,24 @@ foreach ($metaentries['remote'] as $key => $value) {
 	}
 }
 
-
-
-
 $t = new SimpleSAML_XHTML_Template($config, 'core:frontpage_federation.tpl.php');
+
+# look up translated string
+$mtype = array(
+    'saml20-sp-remote' => $t->noop('{admin:metadata_saml20-sp}'),
+    'saml20-sp-hosted' => $t->noop('{admin:metadata_saml20-sp}'),
+    'saml20-idp-remote' => $t->noop('{admin:metadata_saml20-idp}'),
+    'saml20-idp-hosted' => $t->noop('{admin:metadata_saml20-idp}'),
+    'shib13-sp-remote' => $t->noop('{admin:metadata_shib13-sp}'),
+    'shib13-sp-hosted' => $t->noop('{admin:metadata_shib13-sp}'),
+    'shib13-idp-remote' => $t->noop('{admin:metadata_shib13-idp}'),
+    'shib13-idp-hosted' => $t->noop('{admin:metadata_shib13-idp}'),
+    'adfs-sp-remote' => $t->noop('{admin:metadata_adfs-sp}'),
+    'adfs-sp-hosted' => $t->noop('{admin:metadata_adfs-sp}'),
+    'adfs-idp-remote' => $t->noop('{admin:metadata_adfs-idp}'),
+    'adfs-idp-hosted' => $t->noop('{admin:metadata_adfs-idp}'),
+);
+
 $t->data['pageid'] = 'frontpage_federation';
 $t->data['isadmin'] = $isadmin;
 $t->data['loginurl'] = $loginurl;
@@ -127,6 +147,7 @@ $t->data['links_federation'] = $links_federation;
 
 
 $t->data['metaentries'] = $metaentries;
+$t->data['mtype'] = $mtype;
 
 
 $t->show();
